<?php
class AtomicOperationsParserTest extends \Codeception\Test\Unit
{
    protected function getErrorCollector()
    {
        return new class implements Countable {
            public $errors = [];
            public function __invoke($error)
            {
                $this->errors[] = $error;
            }

            public function count(): int
            {
                return count($this->errors);
            }
        };
    }

    /**
     * @dataProvider validInputProvider
     */
    public function testValidInputs(object $input): void
    {
        $collector = $this->getErrorCollector();

        $parser = \JsonApi\Ext\AtomicOperations\OperationParser::create();
        $parser->parse($input, $collector);

        $this->assertCount(0, $collector);
    }

    /**
     * @dataProvider invalidInputProvider
     */
    public function testInvalidInputs(object $input): void
    {
        $collector = $this->getErrorCollector();

        $parser = \JsonApi\Ext\AtomicOperations\OperationParser::create();
        $parser->parse($input, $collector);

        $this->assertGreaterThan(0, count($collector));
    }

    public static function validInputProvider(): array
    {
        return self::loadInputsFromDirectory(TEST_FIXTURES_PATH . '/jsonapi-atomic-operations/valid');
    }

    public static function invalidInputProvider(): array
    {
        return self::loadInputsFromDirectory(TEST_FIXTURES_PATH . '/jsonapi-atomic-operations/invalid');
    }

    protected static function loadInputsFromDirectory(string $directory): array
    {
        $result = [];
        foreach (glob("{$directory}/*.json") as $filename) {
            $result[basename($filename, '.json')] = [
                json_decode(file_get_contents($filename)),
            ];
        }

        return $result;

    }
}
